package test

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/test/md"
	"flag"
	"fmt"
	_ "github.com/go-sql-driver/mysql"
	"gopkg.in/yaml.v3"
	"io/ioutil"
	"os"
	"time"
	"xorm.io/xorm"
	"xorm.io/xorm/log"
)

var Db *xorm.Engine

// 初始化配置文件，将cfg.yml读入到内存
func Init() {
	//用指定的名称、默认值、使用信息注册一个string类型flag。
	path := flag.String("c", "../etc/cfg.yml", "config file")
	//解析命令行参数写入注册的flag里。
	//解析之后，flag的值可以直接使用。
	flag.Parse()
	var (
		c    []byte
		err  error
		conf *md.Config
	)
	if c, err = ioutil.ReadFile(*path); err != nil {
		panic(err)
	}
	//yaml.Unmarshal反序列化映射到Config
	if err = yaml.Unmarshal(c, &conf); err != nil {
		panic(err)
	}
	//数据读入内存
	err = InitDB(&conf.DB)
	if err != nil {
		panic(err.Error())
	}
}

func InitDB(c *md.DBCfg) error {
	var (
		err error
		f   *os.File
	)
	//创建Orm引擎
	if Db, err = xorm.NewEngine("mysql", fmt.Sprintf("%s:%s@tcp(%s)/%s?charset=utf8mb4", c.User, c.Psw, c.Host, c.Name)); err != nil {
		return err
	}
	Db.SetConnMaxLifetime(time.Duration(c.MaxLifetime) * time.Second) //设置最长连接时间
	Db.SetMaxOpenConns(c.MaxOpenConns)                                //设置最大打开连接数
	Db.SetMaxIdleConns(c.MaxIdleConns)                                //设置连接池的空闲数大小
	if err = Db.Ping(); err != nil {                                  //尝试ping数据库
		return err
	}
	if c.ShowLog { //根据配置文件设置日志
		Db.ShowSQL(true)        //设置是否打印sql
		Db.Logger().SetLevel(0) //设置日志等级
		//修改日志文件存放路径文件名是%s.log
		path := fmt.Sprintf(c.Path, c.Name)
		f, err = os.OpenFile(path, os.O_APPEND|os.O_WRONLY|os.O_CREATE, 0777)
		if err != nil {
			os.RemoveAll(c.Path)
			if f, err = os.OpenFile(c.Path, os.O_APPEND|os.O_WRONLY|os.O_CREATE, 0777); err != nil {
				return err
			}
		}
		logger := log.NewSimpleLogger(f)
		logger.ShowSQL(true)
		Db.SetLogger(logger)
	}
	return nil
}
